@@ -13,8 +13,10 @@ void scenario_save_state(buffer *buf);
 
 void scenario_load_state(buffer *buf);
 
-void scenario_settings_save_state(buffer *part1, buffer *part2, buffer *part3, buffer *player_name, buffer *scenario_name);
+void scenario_settings_save_state(
+    buffer *part1, buffer *part2, buffer *part3, buffer *player_name, buffer *scenario_name);
 
-void scenario_settings_load_state(buffer *part1, buffer *part2, buffer *part3, buffer *player_name, buffer *scenario_name);
+void scenario_settings_load_state(
+    buffer *part1, buffer *part2, buffer *part3, buffer *player_name, buffer *scenario_name);
 
 #endif // SCENARIO_SCENARIO_H
