@@ -26,6 +26,7 @@
 #include "window/city.h"
 #include "window/message_dialog.h"
 #include "window/popup_dialog.h"
+#include "window/resource_settings.h"
 #include "window/trade_opened.h"
 
 #define MAX_WIDTH 2032
@@ -35,6 +36,7 @@ static void button_help(int param1, int param2);
 static void button_return_to_city(int param1, int param2);
 static void button_advisor(int advisor, int param2);
 static void button_open_trade(int param1, int param2);
+static void button_show_resource_window(int resource, int param2);
 
 static image_button image_button_help[] = {
     {0, 0, 27, 27, IB_NORMAL, GROUP_CONTEXT_ICONS, 0, button_help, button_none, 0, 0, 1}
@@ -45,8 +47,25 @@ static image_button image_button_return_to_city[] = {
 static image_button image_button_advisor[] = {
     {-4, 0, 24, 24, IB_NORMAL, GROUP_MESSAGE_ADVISOR_BUTTONS, 12, button_advisor, button_none, ADVISOR_TRADE, 0, 1}
 };
+static generic_button generic_button_trade_resource[] = {
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_WHEAT, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_VEGETABLES , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_FRUIT , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_OLIVES , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_VINES , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_MEAT, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_WINE , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_OIL , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_IRON , 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_TIMBER, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_CLAY, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_MARBLE, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_WEAPONS, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_FURNITURE, 0},
+    {0, 0, 101, 27, button_show_resource_window, button_none, RESOURCE_POTTERY, 0}
+};
 static generic_button generic_button_open_trade[] = {
-    {30, 61, 440, 26, button_open_trade, button_none, 0, 0}
+    {30, 56, 440, 26, button_open_trade, button_none, 0, 0}
 };
 
 static struct {
@@ -57,6 +76,7 @@ static struct {
     int focus_button_id;
     int is_scrolling;
     int finished_scroll;
+    int focus_resource;
 } data = {0, 1};
 
 static void init(void)
@@ -109,9 +129,15 @@ static void draw_paneling(void)
 static void draw_trade_resource(resource_type resource, int trade_max, int x_offset, int y_offset)
 {
     graphics_draw_inset_rect(x_offset, y_offset, 26, 26);
+
     int image_id = resource + image_group(GROUP_EMPIRE_RESOURCES);
     int resource_offset = resource_image_offset(resource, RESOURCE_IMAGE_ICON);
     image_draw(image_id + resource_offset, x_offset + 1, y_offset + 1);
+
+    if (data.focus_resource == resource) {
+        button_border_draw(x_offset - 2, y_offset - 2, 101 + 4, 30, 1);
+    }
+
     switch (trade_max) {
         case 15:
             image_draw(image_group(GROUP_TRADE_AMOUNT), x_offset + 21, y_offset - 1);
@@ -128,45 +154,48 @@ static void draw_trade_resource(resource_type resource, int trade_max, int x_off
 static void draw_trade_city_info(const empire_object *object, const empire_city *city)
 {
     int x_offset = (data.x_min + data.x_max - 500) / 2;
-    int y_offset = data.y_max - 108;
+    int y_offset = data.y_max - 113;
     if (city->is_open) {
         // city sells
-        lang_text_draw(47, 10, x_offset + 40, y_offset + 30, FONT_NORMAL_GREEN);
+        lang_text_draw(47, 10, x_offset + 44, y_offset + 40, FONT_NORMAL_GREEN);
         int index = 0;
         for (int resource = RESOURCE_MIN; resource < RESOURCE_MAX; resource++) {
             if (!empire_object_city_sells_resource(object->id, resource)) {
                 continue;
             }
             int trade_max = trade_route_limit(city->route_id, resource);
-            draw_trade_resource(resource, trade_max, x_offset + 100 * index + 120, y_offset + 21);
+            draw_trade_resource(resource, trade_max, x_offset + 104 * index + 120, y_offset + 31);
             int trade_now = trade_route_traded(city->route_id, resource);
             if (trade_now > trade_max) {
                 trade_max = trade_now;
             }
-            int text_width = text_draw_number(trade_now, '@', "", x_offset + 100 * index + 150, y_offset + 30, FONT_NORMAL_GREEN);
-            text_width += lang_text_draw(47, 11, x_offset + 100 * index + 148 + text_width, y_offset + 30, FONT_NORMAL_GREEN);
-            text_draw_number(trade_max, '@', "", x_offset + 100 * index + 138 + text_width, y_offset + 30, FONT_NORMAL_GREEN);
+            int text_width = text_draw_number(trade_now, '@', "",
+                x_offset + 104 * index + 150, y_offset + 40, FONT_NORMAL_GREEN);
+            text_width += lang_text_draw(47, 11,
+                x_offset + 104 * index + 148 + text_width, y_offset + 40, FONT_NORMAL_GREEN);
+            text_draw_number(trade_max, '@', "",
+                x_offset + 104 * index + 138 + text_width, y_offset + 40, FONT_NORMAL_GREEN);
             index++;
         }
         // city buys
-        lang_text_draw(47, 9, x_offset + 40, y_offset + 60, FONT_NORMAL_GREEN);
+        lang_text_draw(47, 9, x_offset + 44, y_offset + 71, FONT_NORMAL_GREEN);
         index = 0;
         for (int resource = RESOURCE_MIN; resource < RESOURCE_MAX; resource++) {
             if (!empire_object_city_buys_resource(object->id, resource)) {
                 continue;
             }
             int trade_max = trade_route_limit(city->route_id, resource);
-            draw_trade_resource(resource, trade_max, x_offset + 100 * index + 120, y_offset + 51);
+            draw_trade_resource(resource, trade_max, x_offset + 104 * index + 120, y_offset + 62);
             int trade_now = trade_route_traded(city->route_id, resource);
             if (trade_now > trade_max) {
                 trade_max = trade_now;
             }
             int text_width = text_draw_number(trade_now, '@', "",
-                                             x_offset + 100 * index + 150, y_offset + 60, FONT_NORMAL_GREEN);
+                                             x_offset + 104 * index + 150, y_offset + 71, FONT_NORMAL_GREEN);
             text_width += lang_text_draw(47, 11,
-                                        x_offset + 100 * index + 148 + text_width, y_offset + 60, FONT_NORMAL_GREEN);
+                                        x_offset + 104 * index + 148 + text_width, y_offset + 71, FONT_NORMAL_GREEN);
             text_draw_number(trade_max, '@', "",
-                             x_offset + 100 * index + 138 + text_width, y_offset + 60, FONT_NORMAL_GREEN);
+                             x_offset + 104 * index + 138 + text_width, y_offset + 71, FONT_NORMAL_GREEN);
             index++;
         }
     } else { // trade is closed
@@ -232,14 +261,14 @@ static void draw_roman_army_info(const empire_object *object)
     if (city_military_distant_battle_roman_army_is_traveling()) {
         if (city_military_distant_battle_roman_months_traveled() == object->distant_battle_travel_months) {
             int x_offset = (data.x_min + data.x_max - 240) / 2;
-            int y_offset = data.y_max - 88;
+            int y_offset = data.y_max - 68;
             int text_id;
             if (city_military_distant_battle_roman_army_is_traveling_forth()) {
                 text_id = 15;
             } else {
                 text_id = 16;
             }
-            lang_text_draw_multiline(47, text_id, x_offset, y_offset, 240, FONT_NORMAL_BLACK);
+            lang_text_draw_multiline(47, text_id, x_offset, y_offset, 240, FONT_NORMAL_GREEN);
         }
     }
 }
@@ -251,7 +280,7 @@ static void draw_enemy_army_info(const empire_object *object)
             lang_text_draw_multiline(47, 14,
                 (data.x_min + data.x_max - 240) / 2,
                 data.y_max - 68,
-                240, FONT_NORMAL_BLACK);
+                240, FONT_NORMAL_GREEN);
         }
     }
 }
@@ -289,8 +318,6 @@ static void draw_background(void)
     if (data.x_min || data.y_min) {
         graphics_clear_screen();
     }
-    draw_paneling();
-    draw_object_info();
 }
 
 static void draw_empire_object(const empire_object *obj)
@@ -317,7 +344,8 @@ static void draw_empire_object(const empire_object *obj)
             city->type == EMPIRE_CITY_FUTURE_ROMAN) {
             image_id = image_group(GROUP_EMPIRE_FOREIGN_CITY);
         } else if (city->type == EMPIRE_CITY_TRADE) {
-            // Fix cases where empire map still gives a blue flag for new trade cities (e.g. Massilia in campaign Lugdunum)
+            // Fix cases where empire map still gives a blue flag for new trade cities
+            // (e.g. Massilia in campaign Lugdunum)
             image_id = image_group(GROUP_EMPIRE_CITY_TRADE);
         }
     }
@@ -358,7 +386,8 @@ static void draw_invasion_warning(int x, int y, int image_id)
 
 static void draw_map(void)
 {
-    graphics_set_clip_rectangle(data.x_min + 16, data.y_min + 16, data.x_max - data.x_min - 32, data.y_max - data.y_min - 136);
+    graphics_set_clip_rectangle(data.x_min + 16, data.y_min + 16,
+        data.x_max - data.x_min - 32, data.y_max - data.y_min - 136);
 
     empire_set_viewport(data.x_max - data.x_min - 32, data.y_max - data.y_min - 136);
 
@@ -393,7 +422,8 @@ static void draw_panel_buttons(const empire_city *city)
     image_buttons_draw(data.x_max - 44, data.y_max - 100, image_button_advisor, 1);
     if (city) {
         if (city->type == EMPIRE_CITY_TRADE && !city->is_open) {
-            button_border_draw((data.x_min + data.x_max - 500) / 2 + 30, data.y_max - 44, 440, 26, data.selected_button);
+            button_border_draw((data.x_min + data.x_max - 500) / 2 + 30, data.y_max - 49, 440,
+                26, data.selected_button);
         }
     }
 }
@@ -411,8 +441,10 @@ static void draw_foreground(void)
             city = empire_city_get(data.selected_city);
         }
     }
+    draw_paneling();
     draw_city_name(city);
     draw_panel_buttons(city);
+    draw_object_info();
 }
 
 static int is_outside_map(int x, int y)
@@ -438,7 +470,7 @@ static void handle_input(const mouse *m, const hotkeys *h)
         empire_scroll_map(position.x, position.y);
     }
     if (m->is_touch) {
-        const touch *t = get_earliest_touch();
+        const touch *t = touch_get_earliest();
         if (!is_outside_map(t->current_point.x, t->current_point.y)) {
             if (t->has_started) {
                 data.is_scrolling = 1;
@@ -452,6 +484,7 @@ static void handle_input(const mouse *m, const hotkeys *h)
         }
     }
     data.focus_button_id = 0;
+    data.focus_resource = 0;
     int button_id;
     image_buttons_handle_mouse(m, data.x_min + 20, data.y_max - 44, image_button_help, 1, &button_id);
     if (button_id) {
@@ -468,13 +501,40 @@ static void handle_input(const mouse *m, const hotkeys *h)
     determine_selected_object(m);
     int selected_object = empire_selected_object();
     if (selected_object) {
-        if (empire_object_get(selected_object -1)->type == EMPIRE_OBJECT_CITY) {
-            data.selected_city = empire_city_get_for_object(selected_object -1);
+        const empire_object *obj = empire_object_get(selected_object - 1);
+        if (obj->type == EMPIRE_OBJECT_CITY) {
+            data.selected_city = empire_city_get_for_object(selected_object - 1);
             const empire_city *city = empire_city_get(data.selected_city);
-            if (city->type == EMPIRE_CITY_TRADE && !city->is_open) {
-                generic_buttons_handle_mouse(
-                    m, (data.x_min + data.x_max - 500) / 2, data.y_max - 105,
+            if (city->type == EMPIRE_CITY_TRADE) {
+                if (city->is_open) {
+                    int x_offset = (data.x_min + data.x_max - 500) / 2;
+                    int y_offset = data.y_max - 113;
+                    int index_sell = 0;
+                    int index_buy = 0;
+
+                    // we only want to handle resource buttons that the selected city trades
+                    for (int resource = RESOURCE_MIN; resource < RESOURCE_MAX; resource++) {
+                        if (empire_object_city_sells_resource(obj->id, resource)) {
+                            generic_buttons_handle_mouse(m, x_offset + 120 + 104 * index_sell, y_offset + 31,
+                                generic_button_trade_resource + resource - 1, 1, &button_id);
+                            index_sell++;
+                        } else if (empire_object_city_buys_resource(obj->id, resource)) {
+                            generic_buttons_handle_mouse(m, x_offset + 120 + 104 * index_buy, y_offset + 62,
+                                generic_button_trade_resource + resource - 1, 1, &button_id);
+                            index_buy++;
+                        }
+
+                        if (button_id) {
+                            data.focus_resource = resource;
+                            // if we're focusing any button we can skip further checks
+                            break;
+                        }
+                    }
+                } else {
+                    generic_buttons_handle_mouse(
+                        m, (data.x_min + data.x_max - 500) / 2, data.y_max - 105,
                         generic_button_open_trade, 1, &data.selected_button);
+                }
             }
         }
         if (input_go_back_requested(m, h)) {
@@ -498,50 +558,34 @@ static int is_mouse_hit(tooltip_context *c, int x, int y, int size)
 static int get_tooltip_resource(tooltip_context *c)
 {
     const empire_city *city = empire_city_get(data.selected_city);
-    if (city->type != EMPIRE_CITY_TRADE) {
+    // we only want to check tooltips on our own closed cities.
+    // open city resource tooltips are handled by their respective buttons directly
+    if (city->type != EMPIRE_CITY_TRADE || city->is_open) {
         return 0;
     }
     int object_id = empire_selected_object() - 1;
     int x_offset = (data.x_min + data.x_max - 500) / 2;
-    int y_offset = data.y_max - 108;
-
-    if (city->is_open) {
-        for (int r = RESOURCE_MIN, index = 0; r < RESOURCE_MAX; r++) {
-            if (empire_object_city_sells_resource(object_id, r)) {
-                if (is_mouse_hit(c, x_offset + 120 + 100 * index, y_offset + 21, 26)) {
-                    return r;
-                }
-                index++;
-            }
-        }
-        for (int r = 1, index = 0; r <= 15; r++) {
-            if (empire_object_city_buys_resource(object_id, r)) {
-                if (is_mouse_hit(c, x_offset + 120 + 100 * index, y_offset + 51, 26)) {
-                    return r;
-                }
-                index++;
-            }
-        }
-    } else {
-        int item_offset = lang_text_get_width(47, 5, FONT_NORMAL_GREEN);
-        for (int r = RESOURCE_MIN; r < RESOURCE_MAX; r++) {
-            if (empire_object_city_sells_resource(object_id, r)) {
-                if (is_mouse_hit(c, x_offset + 60 + item_offset, y_offset + 35, 26)) {
-                    return r;
-                }
-                item_offset += 32;
-            }
-        }
-        item_offset += lang_text_get_width(47, 4, FONT_NORMAL_GREEN);
-        for (int r = 1; r <= 15; r++) {
-            if (empire_object_city_buys_resource(object_id, r)) {
-                if (is_mouse_hit(c, x_offset + 110 + item_offset, y_offset + 35, 26)) {
-                    return r;
-                }
-                item_offset += 32;
+    int y_offset = data.y_max - 113;
+    
+    int item_offset = lang_text_get_width(47, 5, FONT_NORMAL_GREEN);
+    for (int r = RESOURCE_MIN; r < RESOURCE_MAX; r++) {
+        if (empire_object_city_sells_resource(object_id, r)) {
+            if (is_mouse_hit(c, x_offset + 60 + item_offset, y_offset + 33, 26)) {
+                return r;
             }
+            item_offset += 32;
         }
     }
+    item_offset += lang_text_get_width(47, 4, FONT_NORMAL_GREEN);
+    for (int r = RESOURCE_MIN; r <= RESOURCE_MAX; r++) {
+        if (empire_object_city_buys_resource(object_id, r)) {
+            if (is_mouse_hit(c, x_offset + 110 + item_offset, y_offset + 33, 26)) {
+                return r;
+            }
+            item_offset += 32;
+        }
+    }
+
     return 0;
 }
 
@@ -571,7 +615,7 @@ static void get_tooltip_trade_route_type(tooltip_context *c)
 
 static void get_tooltip(tooltip_context *c)
 {
-    int resource = get_tooltip_resource(c);
+    int resource = data.focus_resource ? data.focus_resource : get_tooltip_resource(c);
     if (resource) {
         c->type = TOOLTIP_BUTTON;
         c->text_id = 131 + resource;
@@ -602,6 +646,11 @@ static void button_advisor(int advisor, int param2)
     window_advisors_show_advisor(advisor);
 }
 
+static void button_show_resource_window(int resource, int param2)
+{
+    window_resource_settings_show(resource);
+}
+
 static void confirmed_open_trade(int accepted)
 {
     if (accepted) {
